#!/usr/bin/env python

import sys,types,socket,string,exceptions

try:
	from cStringIO import StringIO
except:
	from StringIO import StringIO

from manipulator import *


######################################################################
# General ostream class
######################################################################

class ostream:

	def __init__(self):
		self.fp 		= None
		self.data		= StringIO()
		self._padding 	= 0
		self.mode 		= 'dec'
		self._floatmode = 'fixed'
		self._casemode  = 'uppercase'
		self._setfill	= ''


	def __lshift__(self,arg):

		if isinstance(arg,MANIPULATOR): 

			# obsolet
			if arg.type in ['dec','hex','oct']:
				self.mode = arg.type
			

			if arg.type in ['flush','endl','ends']:
				if arg.output != None: 	self.data.write(arg.output)
				self.flush_data()
				self.data = StringIO()

			if arg.type in ['precision']:
				self._precision = arg.value

			if arg.type in ['scientific','fixed']:
				self._floatmode = arg.type

			if arg.type in ['uppercase','nouppercase']:
				self._casemode = arg.type

			if arg.type in ['setw']:
				self._padding = arg.value

			if arg.type in ['setfill']:
				self._setfill = arg.value

		elif type(arg) in [types.IntType,types.LongType]:
			if self.mode=='dec':	self.data.write(self.do_pad(str(arg)))
			elif self.mode=='hex':	self.data.write(self.do_pad('%x' % arg))
			elif self.mode=='oct':	self.data.write(self.do_pad('%o' % arg))


		elif type(arg) in [types.FloatType]:
			s = self.long_fmt() % arg 
			if self._casemode=='uppercase':	s = string.upper(s)
			else:							s = string.lower(s)
			self.data.write(self.do_pad(s))


		else:
			self.data.write(self.do_pad(str(arg)))

		return self


	def do_pad(self,arg):
		if self._padding==0:	return arg
		else:
			s = string.ljust(arg,self._padding)
			return  string.replace(s,' ',self._setfill)


	def close(self):
		if self.fp != None and self.fp != sys.stdout:
			self.fp.close()


	def __del__(self):
		self.close()

	def flush_data(self):
		self.fp.write(self.data.getvalue())


	def precision(self):
		return self._precision

	def long_fmt(self):
		fmt = '%%.%d' % self._precision
		if self._floatmode=='fixed':	fmt = fmt + 'f'
		else:							fmt = fmt + 'e'
		return fmt


######################################################################
# derived class for string streams
######################################################################

class StrOstream(ostream):
	def __init__(self):
		ostream.__init__(self)
		self.fp = StringIO()

	def __del__(self): pass
	def __str__(self):	return self.fp.getvalue()


######################################################################
# derived class for file streams
######################################################################

class FileOstream(ostream):
	def __init__(self,fname=None,fp=None):
		ostream.__init__(self)

		if fname != None:		
			self.fp = open(fname,'w')

		elif fp != None:
			self.fp = fp

		else:
			self.fp = sys.stdout


######################################################################
# main()
######################################################################

if __name__=='__main__':
	import math

	num = math.pi
	x=FileOstream()
	x << hex <<16 << endl;

	for x in [FileOstream(),FileOstream('sux.out')]:

		for p in range(0,5):
			x << setprecision(p) << "Precision: " <<  x.precision() << " " << num << endl;

		for p in range(0,15):
			x <<setw(20) <<  setprecision(p) << setfill('%') <<  uppercase << num << setfill(' ') << num << endl; 
	

